#!/usr/bin/env Rscript

# Input arguments ----

Args <- commandArgs(trailingOnly=TRUE)

# Required code ----

if (FALSE) {
  install.packages(pkg=c("devtools", "tidyverse"))
  devtools::install_github("gregorgorjanc/GorjancR")
}
library(tidyverse)
library(GorjancR)
source("Functions.R")

# Population parameters ----
nPop       <-     4
nGen       <-    16
nIndPerPop <- 10000
nSire      <-    25 # see below for change of this with different Ne
nDam       <- nIndPerPop/2
nBaseHap   <- 10000
nRep       <-    10

# Arguments taking effect ----
if (length(Args) > 0) {
  Reps <- Args[1]
} else {
  Reps <- 1:nRep
}
cat("Reps: ", Reps, "\n")

if (length(Args) > 1) {
  Mode <- Args[2]
} else {
  Mode <- "All"
}
cat("Mode: ", Mode, "\n")

if (length(Args) > 2) {
  BaseNe <- as.numeric(Args[3])
} else {
  BaseNe <- 100
}
cat("BaseNe: ", BaseNe, "\n")

if (BaseNe == 10000) {
  nSire <- 5000
}

# Genome parameters ----
nChrom         <-        30
nBpChrom       <- 1.0*10^+8
MutRate        <- 1.0*10^-8
RecRate        <- 1.0*10^-8
if (BaseNe == 100) {
  CompressionPct <- 20
}
if (BaseNe == 10000) {
  CompressionPct <- 10
}
# Ne    nSegSitesPerChrom PerChrom PerGenome
#   100            ~700K  -> ~140K -> ~4200K
# 10000           ~1400K  -> ~140K -> ~4200K

# These are estimates of Ne for different spans of generations (MacLeod et al., 2013; http://mbe.oxfordjournals.org/content/30/9/2209)
MacLeodNe <- data.frame(GenSpan=c(    3,    3,    6,    6,    6,  130,  300,  200, 1100,   600,  1000, 29800, 900000),
                        # Years    2000  1985  1970  1940  1930  1900   ...
                        #          1986  1971  1941  1931  1901  1251   ...
                           NeMean=c( 90,  120,  250,  350, 1000, 1500, 2000, 2500, 3500,  7000, 10000, 17000,  62000),
                         NeApprox=c(100,  150,  300,  450, 1250, 2000, 3000, 4000, 5000, 10000, 15000, 25000, 100000))
# cumsum(MacLeodNe[["GenSpan"]])

MacLeodNe[["GenerationsAgo"]] <- cumsum(MacLeodNe[["GenSpan"]])
MacLeodNe[["GenerationsAgo"]][2:nrow(MacLeodNe)] <- MacLeodNe[["GenerationsAgo"]][1:(nrow(MacLeodNe)-1)]
MacLeodNe <- filter(MacLeodNe, NeApprox > BaseNe)
OldNeFlags <- paste("-eN", MacLeodNe[["GenerationsAgo"]] / (4 * BaseNe), MacLeodNe[["NeApprox"]] / BaseNe, collapse=" ")

nBpChromFlag <- formatC(nBpChrom, format="E")
MutRateFlag <- paste0("-t ", formatC(4 * BaseNe * MutRate, format="E"))
RecRateFlag <- paste0("-r ", formatC(4 * BaseNe * RecRate, format="E"))

nPopFlag <- paste("-I", nPop, paste(rep(nBaseHap, times=nPop), collapse=" "))
nPopSplitFlag <- paste(paste("-ej ",  25/(4 * BaseNe), "2 1"),
                       paste("-ej ",  50/(4 * BaseNe), "3 1"),
                       paste("-ej ", 100/(4 * BaseNe), "4 1"))

# Create a standalone directory with all the data ----
dir.create("Data", showWarnings=FALSE)
setwd("Data")
try(SetupFiles(Platform=TRUE))

for (Rep in Reps) {
  # Rep <- 1

  Dir <- paste0("BaseNe_", BaseNe, "_Rep_", Rep)
  # unlink(Dir, recursive=TRUE)
  dir.create(Dir, showWarnings=FALSE)
  RepOldDir <- setwd(Dir)
  try(SetupFiles())

  # Base population haplotypes ----

  if (grepl(x=Mode, pattern="Chrom") | grepl(x=Mode, pattern="All")) {
    nChromTmp <- nChrom
    ChromTmp <- NULL
    if (grepl(x=Mode, pattern="Chrom")) {
      nChromTmp <- NULL
      ChromTmp <- sub(x=Mode, pattern="Chrom", replacement="")
      if (ChromTmp == "All") {
        nChromTmp <- nChrom
        ChromTmp <- NULL
      }
    }
    SimulateBaseHaplotypes(nChrom=nChromTmp, Chrom=ChromTmp,
                           MaCSParameters=paste(nBaseHap*nPop, nBpChromFlag, MutRateFlag, RecRateFlag,
                                                OldNeFlags, nPopFlag, nPopSplitFlag))
  }

  # The reference population (keeping it smallish to run fast and get genome parameters etc.) ----
  if (grepl(x=Mode, pattern="BasePop") | grepl(x=Mode, pattern="All")) {
    SimulatePopulation(nGen=0, nIndPerPop, nSire, nDam, nChrom, nBaseHap, Compression=TRUE,
    #SimulatePopulation(nGen=0, nIndPerPop, nSire, nDam, nChrom, nBaseHap, Compression=FALSE,
                       CompressionPct=CompressionPct, ChipAlleleFreq="0.05,0.95")
    System(Command="rm -rf SimulatedData_Population_Reference")
    System(Command="mv -f SimulatedData SimulatedData_Population_Reference")
    for (Chrom in 1:nChrom) {
      System(Command=paste0("cd Chromosomes/Chromosome", Chrom, ";",
                            "mv -f MacsHaplotypesCompressed.txt MacsHaplotypesCompressedPop0.txt;",
                            #"mv -f MacsHaplotypes.txt MacsHaplotypesCompressedPop0.txt;",
                            "split -l ", nBaseHap, " MacsHaplotypesCompressedPop0.txt;",
                            "mv -f xaa MacsHaplotypesCompressedPop1.txt;",
                            "mv -f xab MacsHaplotypesCompressedPop2.txt;",
                            "mv -f xac MacsHaplotypesCompressedPop3.txt;",
                            "mv -f xad MacsHaplotypesCompressedPop4.txt;"))
    }
  }

  # Subpopulations ----

  if (grepl(x=Mode, pattern="SceA") | grepl(x=Mode, pattern="All")) {
    for (Pop in 1:nPop) {
      # Pop <- 1
      if (grepl(x=Mode, pattern=paste0("SceA", Pop)) | grepl(x=Mode, pattern="All")) {
        Dir <- paste0("Scenario_A", Pop)
        unlink(Dir, recursive=TRUE)
        dir.create(Dir)
        OldScenarioDir <- setwd(Dir)
        SetupFiles()
        # Take the same base haplotypes for each population
        HandleChromosomesFolder(ChromosomesFolder="../Chromosomes",
                                MacsHaplotypesFile="MacsHaplotypesCompressedPop1.txt",
                                CompressedBefore=TRUE)
                                #CompressedBefore=FALSE)
        SimulatePopulation(nGen, nIndPerPop, nSire, nDam, nChrom, nBaseHap,
                           UseOldSnpLocations=TRUE, UseOldQTNYesNo=TRUE, UseOldEffectsYesNo=TRUE,
                           OldFolder="../SimulatedData_Population_Reference")
        System(Command="rm -rf Chromosomes")
        setwd(OldScenarioDir)
      }
    }
  }

  setwd(RepOldDir)
}
